/*
Interface.h

  Functions for interfacing with the Adaboy system to be used by the application
  developer to use the patented Adaboy advertising system.

  Copyright Adaboy Inc.

  Revision:  Beta  0.10
  Rev Date:  05/01/2000
*/

// Include this file after your basic program includes like windows.h etc

// this section file defines the data types used by the client API interface
// It was included in 2 separate files but this makes it easier to include

// if you change these constants, it will have no effect on the system but your
// program may not work properly

#define FILENAME_WITH_PATH_SIZE       128
#define FILENAME_IMG_PATH_SIZE         16

#define USER_DEMOG_NAME_LENGTH         32
#define USER_DEMOG_PW_LENGTH           32

#define MIN_LOGIN_NAME_LEN              8
#define MIN_LOGIN_NAME_DIGITS           2
#define MIN_LOGIN_NAME_CHARS            5

#define MIN_LOGIN_PW_LEN                5

#define MAX_CLIENT_ADS				             32     

	  
struct stAdaboyUser
{
 char LoginName[USER_DEMOG_NAME_LENGTH];
 char DescriptName[USER_DEMOG_NAME_LENGTH];   // optional for local ID of the user by your program
                                             // this information is NOT transmitted to Adaboy
};

struct stImageData
{
 int ID;
 int w,h;             // width and height
};


#define CFF_NONE 0
#define CFF_JPG  1
#define CFF_BMP  2

#ifdef __cplusplus 
extern "C"
{
#endif

/*
   Initialize the Adaboy Client module and prepare to use functions
   Call this function before calling any other functions
   Returns 1 on success, 0 on failure to start thread.

			You must pass your developer ID and your application IDs to this function.
			Application ID must not be 0.  If you fail to use the correct ID that
			was assigned to you, you may not be paid for the views generated.
			The version is for your use, use it to differentiate builds or ???
*/
int ABC_Initialize(WORD developerid,WORD ApplicationID,WORD version);         // prepares for operation, sets up data

// pass this function the IP and server name ( under 32 chars ) so that you can
// tell how popular each server or game is.  The name is a developer defined text field
// that can be included in aggregate reports along with the Game server IP
int ABC_SetGameServerName(unsigned int GameServerIP,char *name);


// ABC_Ready returns true when the Adaboy library is ready for communication
// with the server.  
int ABC_Ready();              // enquire if the client is ready to go

// returns 0 on success, non-zero on failure to close socket subsystem
int ABC_ShutDown();           // cleanup, kills threads etc.

char *ABC_GetVersion();                 // returns a pointer to the printable version string ( static, do NOT free )

/*
   User list functions that manipulate the userdata 
*/

// returns a pointer to an allocated list of users, remember to free the memory after use
struct stAdaboyUser *ABC_UserGetList(int *num);  // grab list of users in machine's list

// reurns 1 on success, 0 on failure.  Fails for bad user number
int ABC_UserSelect(unsigned int num,int adquant); // log in as the 'num' user from the list

// returns the user number of the selected user.  Pass the returned value to ABC_UserSelect
// call this with your game's main window handle or 0 if you don't have a window handle
int SelectUserBuiltInDialog(HWND parent);         // select user using built in dialog boxes

// Add a user that has already signed up at the Adaboy website to the local machine
int ABC_AddExistingUser(char *login,char *pwd);   // adds an existing user to the registry 

// if the user functions report a problem, call this function to get a text decription of the problem.
int ABC_UserGetError(char *errtxt,int maxlen);  // gets the error caused by the validate or create call


// call this function with the address of a pointer to a message and a URL
// they will be set to either NULL if there isn't any message and if there
// is a message, they will be set to some malloc'd strings.  The Adaboy client
// will then 'unremember' them.  remember to free the memory returned.
// Messages are set by the login and logout functions, ABC_UserSelect and ABC_SessionEnd
int ABC_GetUserMessage(char **message,char **URL); // returns true if there is a user message

// this function checks the return value from the server to see if the game has been superceeded
// if it has, a message will have been sent for some time before this goes false
int ABC_GetGameValid();   // if you have 'expired' the game, this will return false


/*
   acquisition and image retrieval functions
*/
// returns number of images ready at return.  May return less than requested
// due to timeout, communication problems or selection of fewer images
// remember to use the value returned to display images
int ABC_ForceImagesReady(int quantity,int timeout);         // delays return until quantity images are ready.  Timeout in seconds

int ABC_ImageGetData(int quant,struct stImageData *data);   // get a pointer to a block of images. returns number returned and stores the
                                                            // data into the memory block passed to the program

// query the client to determine the number of images ready.  It may be
// used to detect the availability of additional images.
int ABC_ImagesReady();  // Returns the number of images ready right now

// These functions retrieve the images from the client.
char *ABC_ImageGetMem(int ID,int Format,int *len);       // returns image file as an allocated memory block.  Format = 0 for JPG
FILE *ABC_ImageGetFile(int ID);       // returns a file pointer to image on disk
char *ABC_ImageGetFileName(int ID);   // returns file name with full path  ( May have spaces in the filename )


/* 
  Stats reporting functions  etc   chkesp

  These are still subject to change...

*/

// call this function to report the number of times the image was displayed.
// When you present the ad to the user, call this function.  
// The target display rate is around 1 ad per minute.
// Please insure that the user is shown the images you report.  
// Your game will be auditted by Adaboy.
int ABC_ImageView(int ID,int number); 

// implicit call by select user.  Call only if the user has been selected but a new session is starting
int ABC_SessionBegin(int adquant);

int ABC_SessionLevel(int LevelID); 

int ABC_SessionEnd();

int ABC_SessionEndEx();  // Additional reporting to be added...     

int ABC_Error(char *text,int maxlen);   // get indication of problems

#ifdef __cplusplus 
                                                                                                     
}
#endif





