/*
text.cpp
text graphics routines ...

Copyright (C) 1995-2005  Everett Bell

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "globals.h"



int getlength( char *text )
{
    int lettercount = 0;
    int length = 0;

	MYFONT *fontptr;


	fontptr = &myfonttype[myfont];

    while( text[lettercount] )
    {
// TODO -- ADD in special case for [ and ] characters too !!        
		if( text[lettercount] == 126 ) { length += 6; lettercount++; continue; }	// ??
		if( text[lettercount] == 94 ) { lettercount += 2; continue; }	// Carat ESCAPE code

		length += fontptr->letter[ (text[lettercount]-32) ].width - 1;

        lettercount++;

    } // end lettercount loop

    return length;

} // end getlength



void showtext( char *text, int x, int y )
{
    int index = 0;      // Character counter
    int templength;

	MYFONT *fontptr;

	fontptr = &myfonttype[myfont];

// Hold in bounds of screen if needed ... 12/4/96.

        if( textinbounds )
                {
                if( y < 6 ) y = 6;
                if( y > SCREENHEIGHT-fontptr->height ) y = SCREENHEIGHT-fontptr->height;
                if( x < 6 ) x = 6;

                templength = getlength( text );

                if( (x + templength) > SCREENWIDTH-6 ) x = SCREENWIDTH-6 - templength;
                } // end if textinbounds

        else if( y > SCREENHEIGHT ) y = SCREENHEIGHT;

    do {
		if( text[index] == '^' )
		{
			index++;
			switch( (text[index]%8) )
			{
			default:
				sprintf( debugstr, "Unrecognized ESCAPE code in string ... ^%c\n", text[index] );
				WRITETODEBUGFILE;

			case 0:
				mytextcolor = TEXTRED;		break;
			case 1:
				mytextcolor = TEXTORANGE;	break;
			case 2:
				mytextcolor = TEXTYELLOW;	break;
			case 3:
				mytextcolor = TEXTGREEN;	break;
			case 4:
				mytextcolor = TEXTCYAN;		break;
			case 5:
				mytextcolor = TEXTBLUE;		break;
			case 6:
				mytextcolor = TEXTPURPLE;	break;
			case 7:
				mytextcolor = TEXTWHITE;	break;

			} // end switch on text[index]

			index++;
			continue;

		} // end ESCAPE code

        showchar( text[index], x, y, 0 );
// Tilde ...
        if( text[index] == 126 ) { showchar( '-', x, y-4, 0 ); index++; x+=6; continue; }

		x += fontptr->letter[ text[index]-32 ].width - 1;
        index++;
    } while ( text[index] != '\x0' );

} // end showtext



void showtextjr( char *text, int x, int y )
{
    int index = 0;      // Character counter
    int templength;

	MYFONT *fontptr;

	fontptr = &myfonttype[myfont];

	x -= getlength( text );

// Hold in bounds of screen if needed ... 12/4/96.

        if( textinbounds )
                {
                if( y < 6 ) y = 6;
                if( y > SCREENHEIGHT-fontptr->height ) y = SCREENHEIGHT-fontptr->height;
                if( x < 6 ) x = 6;

                templength = getlength( text );

                if( (x + templength) > SCREENWIDTH-6 ) x = SCREENWIDTH-6 - templength;
                } // end if textinbounds

        else if( y > SCREENHEIGHT ) y = SCREENHEIGHT;


    do {
        showchar( text[index], x, y, 0 );
// Tilde ...
        if( text[index] == 126 ) { showchar( '-', x, y-4, 0 ); index++; x+=6; continue; }

		x += fontptr->letter[ text[index]-32 ].width - 1;
        index++;
    } while ( text[index] != '\x0' );

} // end showtextjr



void centertext( char *text, int y, int min, int max )
{
    int index = 0;      // Character counter
    int x;

	MYFONT *fontptr;


	fontptr = &myfonttype[myfont];

    x = (((max-min) - (getlength(text))) >> 1) + min;

    do {
        showchar( text[index], x, y, 0 );
// Tilde ...
        if( text[index] == 126 ) { showchar( '-', x, y-4, 0 ); index++; x+=6; continue; }

		x += fontptr->letter[ text[index]-32 ].width - 1;
        index++;
    } while ( text[index] != '\x0' );

} // end centertext



//
//
// 16bit version of showchar() ...
//
//

/*

void showchar( char c, int x, int y, char recessed )
{
    int index, index2, charlength;
	int height, width;
    byte brightness;
    long offset;
    byte *srcptr;	// Font raw pic data
	word *screenptr;

	dword offsetwidth;

	word r, g, b;
	word rdest, gdest, bdest;
	word destpixel;

	word srcamt, destamt;	// Amount to blend (based on actual font pic data)

	MYFONT *fontptr;



        if( recessed )
                {
                x += 2;
                y += 2;
                } // end if recessed

	    c -= 32;                           // Convert to MY letter format

// Do special characters ...

// TODO -- Make this conform to 16 bit !!

        switch( c )
                {
                case 59:        // Brighten    '['
//                        mytextcolor -= 4;
                        return;
                case 61:        // Darken      ']'
//                        mytextcolor += 4;
                        return;
                }

// If not a character, don't draw !

        if( c > 90 ) return;


		fontptr = &myfonttype[myfont];
		offset = fontptr->letter[c].offset;
		charlength = fontptr->letter[c].width;
		height = fontptr->height;
		width = fontptr->width;



        srcptr = fontptr->picptr + offset;


		LOCKVS;

// TODO -- Change this to reflect pointer to status bar if drawingtomenu ... 
// also need a different lineoffset (640 instead of 768)
//		screenptr = g_vsptr + VSOFFSET + (x+times768[y]);
		if( drawtomenu )
		{
			screenptr = menubarptr + (y<<9) + (y<<7) + x;
			offsetwidth = SCREENWIDTH;
		}
		else
		{
			screenptr = g_vsptr + VSOFFSET + (x+((y<<9)+(y<<8)));
			offsetwidth = VSCREENWIDTH;
		}



		for( index = 0; index < height; index++ )
		{
			for( index2 = 0; index2 < charlength; index2++ )
			{
				brightness = *srcptr;
				if( !brightness )
				{
					screenptr++;
					srcptr++;
					continue;
				}

				// Gonna multiply it for now to expand my old (1-8) fonts to (4-32)
				// Maybe I'll make the new fonts take advantage of this when I make 'em ...
				srcamt = (brightness+1)<<2;
				destamt = 32-srcamt;

				destpixel = *screenptr;

				rdest = (destpixel & rmask) >> rbitshift;
				gdest = (destpixel & gmask) >> gbitshift;
				bdest = (destpixel & bmask);

				r = (textr*srcamt) + (rdest*destamt);
				g = (textg*srcamt) + (gdest*destamt);
				b = (textb*srcamt) + (bdest*destamt);

				r = r>>5;
				g = g>>5;
				b = b>>5;

				*screenptr = (r<<rbitshift) | (g<<gbitshift) | b;

// TODO -- Take this out of the loop !
				if( myfont == ARIAL )
				{
					if( drawtomenu ) *(screenptr+SCREENWIDTH+1) = (word)0;
					else *(screenptr+VSCREENWIDTH+1) = (word)0;
				}

				screenptr++;
				srcptr++;
				
			} // end bit loop

			srcptr += width - charlength;
			screenptr += offsetwidth - charlength;

		} // end line loop

		UNLOCKVS;

		return;

} // end showchar

*/




//
//
// 8bit version of showchar() ...
//
//


void showchar( char c, int x, int y, char recessed )
{
//    int index, index2;
	int charlength;
	int height, width;
//    byte brightness;
    long offset;
//    byte *srcptr;	// Font raw pic data
//	word *screenptr;

//	dword offsetwidth;


//	word destpixel;

	MYFONT *fontptr;


        if( recessed )
                {
                x += 2;
                y += 2;
                } // end if recessed

	    c -= 32;                           // Convert to MY letter format

// Do special characters ...

// TODO -- Make this conform to 16 bit !!

        switch( c )
                {
                case 59:        // Brighten    '['
//                        mytextcolor -= 4;
                        return;
                case 61:        // Darken      ']'
//                        mytextcolor += 4;
                        return;
                }

// If not a character, don't draw !

        if( c > 90 ) return;


		fontptr = &myfonttype[myfont];
		offset = fontptr->letter[c].offset;
		charlength = fontptr->letter[c].width;
		height = fontptr->height;
		width = fontptr->width;



		SelectObject( tempdc, hbmp[fontptr->surfaceno] );

// Transparent!

		// Create mask ...
		SetBkColor( tempdc, RGB(0,0,0) );
		BitBlt( maskdc, 0, 0, charlength, height, tempdc, offset, (mytextcolor<<4)+(mytextcolor<<3), SRCCOPY );

		// Do the work ...
		BitBlt( memdc, x, y, charlength, height, tempdc, offset, (mytextcolor<<4)+(mytextcolor<<3), SRCINVERT );
		BitBlt( memdc, x, y, charlength, height, maskdc, 0, 0, SRCAND );
		BitBlt( memdc, x, y, charlength, height, tempdc, offset, (mytextcolor<<4)+(mytextcolor<<3), SRCINVERT );



//		BitBlt( memdc, x, y, charlength, height, tempdc, offset, mytextcolor*24, SRCCOPY );

		return;

} // end showchar










void inittext( void )
{
//        int index;

		MYFONT *fontptr;


// EURO TEXT ...

// Allocate memory and load in data ...

		fontptr = &myfonttype[EURO]; 
		
//		fontptr->picptr = unpackpcx( "fonts\\arial.pcx" );
		fontptr->surfaceno = DDS_EUROFONT;

		fontptr->height = 24;
		fontptr->width = 800;


        fontptr->letter[0].offset = 790;
        fontptr->letter[0].width = 8;

        fontptr->letter[1].offset = 0;
        fontptr->letter[1].width =  5;
        fontptr->letter[2].offset = 5;
        fontptr->letter[2].width =  8;
        fontptr->letter[3].offset = 13;
        fontptr->letter[3].width =  14;
        fontptr->letter[4].offset = 27;
        fontptr->letter[4].width =  15;
        fontptr->letter[5].offset = 42;
        fontptr->letter[5].width =  19;
        fontptr->letter[6].offset = 61;
        fontptr->letter[6].width =  16;
        fontptr->letter[7].offset = 77;
        fontptr->letter[7].width =  4;
        fontptr->letter[8].offset = 81;
        fontptr->letter[8].width =  7;
        fontptr->letter[9].offset = 88;
        fontptr->letter[9].width =  7;
        fontptr->letter[10].offset = 95;
        fontptr->letter[10].width =  10;
        fontptr->letter[11].offset = 105;
        fontptr->letter[11].width =  11;
        fontptr->letter[12].offset = 116;
        fontptr->letter[12].width =  5;
        fontptr->letter[13].offset = 121;
        fontptr->letter[13].width =  7;
        fontptr->letter[14].offset = 128;
        fontptr->letter[14].width =  5;
        fontptr->letter[15].offset = 133;
        fontptr->letter[15].width =  15;

        fontptr->letter[16].offset = 148;
        fontptr->letter[16].width =  14;
        fontptr->letter[17].offset = 162;
        fontptr->letter[17].width =  11;
        fontptr->letter[18].offset = 173;
        fontptr->letter[18].width =  13;
        fontptr->letter[19].offset = 186;
        fontptr->letter[19].width =  14;
        fontptr->letter[20].offset = 200;
        fontptr->letter[20].width =  15;
        fontptr->letter[21].offset = 215;
        fontptr->letter[21].width =  14;
        fontptr->letter[22].offset = 229;
        fontptr->letter[22].width =  14;
        fontptr->letter[23].offset = 243;
        fontptr->letter[23].width =  13;
        fontptr->letter[24].offset = 256;
        fontptr->letter[24].width =  14;
        fontptr->letter[25].offset = 270;
        fontptr->letter[25].width =  13;

        fontptr->letter[26].offset = 283;
        fontptr->letter[26].width =  5;
        fontptr->letter[27].offset = 288;
        fontptr->letter[27].width =  5;
        fontptr->letter[28].offset = 283;
        fontptr->letter[28].width =  12;
        fontptr->letter[29].offset = 305;
        fontptr->letter[29].width =  11;
        fontptr->letter[30].offset = 316;
        fontptr->letter[30].width =  12;
        fontptr->letter[31].offset = 328;
        fontptr->letter[31].width =  12;
        fontptr->letter[32].offset = 340;
        fontptr->letter[32].width =  15;

        fontptr->letter[33].offset = 355;
        fontptr->letter[33].width =  16;
        fontptr->letter[34].offset = 371;
        fontptr->letter[34].width =  14;
        fontptr->letter[35].offset = 385;
        fontptr->letter[35].width =  14;
        fontptr->letter[36].offset = 399;
        fontptr->letter[36].width =  15;
        fontptr->letter[37].offset = 414;
        fontptr->letter[37].width =  12;
        fontptr->letter[38].offset = 426;
        fontptr->letter[38].width =  11;
        fontptr->letter[39].offset = 437;
        fontptr->letter[39].width =  15;
        fontptr->letter[40].offset = 452;
        fontptr->letter[40].width =  14;
        fontptr->letter[41].offset = 466;
        fontptr->letter[41].width =  5;
        fontptr->letter[42].offset = 471;
        fontptr->letter[42].width =  12;
        fontptr->letter[43].offset = 483;
        fontptr->letter[43].width =  15;
        fontptr->letter[44].offset = 498;
        fontptr->letter[44].width =  12;
        fontptr->letter[45].offset = 510;
        fontptr->letter[45].width =  19;
        fontptr->letter[46].offset = 529;
        fontptr->letter[46].width =  16;
        fontptr->letter[47].offset = 545;
        fontptr->letter[47].width =  15;
        fontptr->letter[48].offset = 560;
        fontptr->letter[48].width =  13;
        fontptr->letter[49].offset = 573;
        fontptr->letter[49].width =  16;
        fontptr->letter[50].offset = 589;
        fontptr->letter[50].width =  14;
        fontptr->letter[51].offset = 603;
        fontptr->letter[51].width =  14;
        fontptr->letter[52].offset = 617;
        fontptr->letter[52].width =  14;
        fontptr->letter[53].offset = 631;
        fontptr->letter[53].width =  14;
        fontptr->letter[54].offset = 645;
        fontptr->letter[54].width =  15;
        fontptr->letter[55].offset = 660;
        fontptr->letter[55].width =  24;
        fontptr->letter[56].offset = 684;
        fontptr->letter[56].width =  16;
        fontptr->letter[57].offset = 700;
        fontptr->letter[57].width =  15;
        fontptr->letter[58].offset = 715;
        fontptr->letter[58].width =  14;

        fontptr->letter[59].offset = 620;
        fontptr->letter[59].width =  1; // SPECIAL
        fontptr->letter[60].offset = 574;
        fontptr->letter[60].width =  5;
        fontptr->letter[61].offset = 620;
        fontptr->letter[61].width =  1; // SPECIAL
        fontptr->letter[62].offset = 583;
        fontptr->letter[62].width =  1;
        fontptr->letter[63].offset = 590;
        fontptr->letter[63].width =  8;
        fontptr->letter[64].offset = 598;
        fontptr->letter[64].width =  4;


        fontptr->letter[65].offset = 355;
        fontptr->letter[65].width =  16;
        fontptr->letter[66].offset = 371;
        fontptr->letter[66].width =  14;
        fontptr->letter[67].offset = 385;
        fontptr->letter[67].width =  14;
        fontptr->letter[68].offset = 399;
        fontptr->letter[68].width =  15;
        fontptr->letter[69].offset = 414;
        fontptr->letter[69].width =  12;
        fontptr->letter[70].offset = 426;
        fontptr->letter[70].width =  11;
        fontptr->letter[71].offset = 437;
        fontptr->letter[71].width =  15;
        fontptr->letter[72].offset = 452;
        fontptr->letter[72].width =  14;
        fontptr->letter[73].offset = 466;
        fontptr->letter[73].width =  5;
        fontptr->letter[74].offset = 471;
        fontptr->letter[74].width =  12;
        fontptr->letter[75].offset = 483;
        fontptr->letter[75].width =  15;
        fontptr->letter[76].offset = 498;
        fontptr->letter[76].width =  12;
        fontptr->letter[77].offset = 510;
        fontptr->letter[77].width =  19;
        fontptr->letter[78].offset = 529;
        fontptr->letter[78].width =  16;
        fontptr->letter[79].offset = 545;
        fontptr->letter[79].width =  15;
        fontptr->letter[80].offset = 560;
        fontptr->letter[80].width =  13;
        fontptr->letter[81].offset = 573;
        fontptr->letter[81].width =  16;
        fontptr->letter[82].offset = 589;
        fontptr->letter[82].width =  14;
        fontptr->letter[83].offset = 603;
        fontptr->letter[83].width =  14;
        fontptr->letter[84].offset = 617;
        fontptr->letter[84].width =  14;
        fontptr->letter[85].offset = 631;
        fontptr->letter[85].width =  14;
        fontptr->letter[86].offset = 645;
        fontptr->letter[86].width =  15;
        fontptr->letter[87].offset = 660;
        fontptr->letter[87].width =  24;
        fontptr->letter[88].offset = 684;
        fontptr->letter[88].width =  16;
        fontptr->letter[89].offset = 700;
        fontptr->letter[89].width =  15;
        fontptr->letter[90].offset = 715;
        fontptr->letter[90].width =  14;


// EURODARK TEXT ...

// Allocate memory and load in data ...

		fontptr = &myfonttype[EURODARK]; 
		
//		fontptr->picptr = unpackpcx( "fonts\\arial.pcx" );
		fontptr->surfaceno = DDS_EUROFONTDARK;

		fontptr->height = 24;
		fontptr->width = 800;


        fontptr->letter[0].offset = 790;
        fontptr->letter[0].width = 8;

        fontptr->letter[1].offset = 0;
        fontptr->letter[1].width =  5;
        fontptr->letter[2].offset = 5;
        fontptr->letter[2].width =  8;
        fontptr->letter[3].offset = 13;
        fontptr->letter[3].width =  14;
        fontptr->letter[4].offset = 27;
        fontptr->letter[4].width =  15;
        fontptr->letter[5].offset = 42;
        fontptr->letter[5].width =  19;
        fontptr->letter[6].offset = 61;
        fontptr->letter[6].width =  16;
        fontptr->letter[7].offset = 77;
        fontptr->letter[7].width =  4;
        fontptr->letter[8].offset = 81;
        fontptr->letter[8].width =  7;
        fontptr->letter[9].offset = 88;
        fontptr->letter[9].width =  7;
        fontptr->letter[10].offset = 95;
        fontptr->letter[10].width =  10;
        fontptr->letter[11].offset = 105;
        fontptr->letter[11].width =  11;
        fontptr->letter[12].offset = 116;
        fontptr->letter[12].width =  5;
        fontptr->letter[13].offset = 121;
        fontptr->letter[13].width =  7;
        fontptr->letter[14].offset = 128;
        fontptr->letter[14].width =  5;
        fontptr->letter[15].offset = 133;
        fontptr->letter[15].width =  15;

        fontptr->letter[16].offset = 148;
        fontptr->letter[16].width =  14;
        fontptr->letter[17].offset = 162;
        fontptr->letter[17].width =  11;
        fontptr->letter[18].offset = 173;
        fontptr->letter[18].width =  13;
        fontptr->letter[19].offset = 186;
        fontptr->letter[19].width =  14;
        fontptr->letter[20].offset = 200;
        fontptr->letter[20].width =  15;
        fontptr->letter[21].offset = 215;
        fontptr->letter[21].width =  14;
        fontptr->letter[22].offset = 229;
        fontptr->letter[22].width =  14;
        fontptr->letter[23].offset = 243;
        fontptr->letter[23].width =  13;
        fontptr->letter[24].offset = 256;
        fontptr->letter[24].width =  14;
        fontptr->letter[25].offset = 270;
        fontptr->letter[25].width =  13;

        fontptr->letter[26].offset = 283;
        fontptr->letter[26].width =  5;
        fontptr->letter[27].offset = 288;
        fontptr->letter[27].width =  5;
        fontptr->letter[28].offset = 283;
        fontptr->letter[28].width =  12;
        fontptr->letter[29].offset = 305;
        fontptr->letter[29].width =  11;
        fontptr->letter[30].offset = 316;
        fontptr->letter[30].width =  12;
        fontptr->letter[31].offset = 328;
        fontptr->letter[31].width =  12;
        fontptr->letter[32].offset = 340;
        fontptr->letter[32].width =  15;

        fontptr->letter[33].offset = 355;
        fontptr->letter[33].width =  16;
        fontptr->letter[34].offset = 371;
        fontptr->letter[34].width =  14;
        fontptr->letter[35].offset = 385;
        fontptr->letter[35].width =  14;
        fontptr->letter[36].offset = 399;
        fontptr->letter[36].width =  15;
        fontptr->letter[37].offset = 414;
        fontptr->letter[37].width =  12;
        fontptr->letter[38].offset = 426;
        fontptr->letter[38].width =  11;
        fontptr->letter[39].offset = 437;
        fontptr->letter[39].width =  15;
        fontptr->letter[40].offset = 452;
        fontptr->letter[40].width =  14;
        fontptr->letter[41].offset = 466;
        fontptr->letter[41].width =  5;
        fontptr->letter[42].offset = 471;
        fontptr->letter[42].width =  12;
        fontptr->letter[43].offset = 483;
        fontptr->letter[43].width =  15;
        fontptr->letter[44].offset = 498;
        fontptr->letter[44].width =  12;
        fontptr->letter[45].offset = 510;
        fontptr->letter[45].width =  19;
        fontptr->letter[46].offset = 529;
        fontptr->letter[46].width =  16;
        fontptr->letter[47].offset = 545;
        fontptr->letter[47].width =  15;
        fontptr->letter[48].offset = 560;
        fontptr->letter[48].width =  13;
        fontptr->letter[49].offset = 573;
        fontptr->letter[49].width =  16;
        fontptr->letter[50].offset = 589;
        fontptr->letter[50].width =  14;
        fontptr->letter[51].offset = 603;
        fontptr->letter[51].width =  14;
        fontptr->letter[52].offset = 617;
        fontptr->letter[52].width =  14;
        fontptr->letter[53].offset = 631;
        fontptr->letter[53].width =  14;
        fontptr->letter[54].offset = 645;
        fontptr->letter[54].width =  15;
        fontptr->letter[55].offset = 660;
        fontptr->letter[55].width =  24;
        fontptr->letter[56].offset = 684;
        fontptr->letter[56].width =  16;
        fontptr->letter[57].offset = 700;
        fontptr->letter[57].width =  15;
        fontptr->letter[58].offset = 715;
        fontptr->letter[58].width =  14;

        fontptr->letter[59].offset = 620;
        fontptr->letter[59].width =  1; // SPECIAL
        fontptr->letter[60].offset = 574;
        fontptr->letter[60].width =  5;
        fontptr->letter[61].offset = 620;
        fontptr->letter[61].width =  1; // SPECIAL
        fontptr->letter[62].offset = 583;
        fontptr->letter[62].width =  7;
        fontptr->letter[63].offset = 590;
        fontptr->letter[63].width =  8;
        fontptr->letter[64].offset = 598;
        fontptr->letter[64].width =  4;


        fontptr->letter[65].offset = 355;
        fontptr->letter[65].width =  16;
        fontptr->letter[66].offset = 371;
        fontptr->letter[66].width =  14;
        fontptr->letter[67].offset = 385;
        fontptr->letter[67].width =  14;
        fontptr->letter[68].offset = 399;
        fontptr->letter[68].width =  15;
        fontptr->letter[69].offset = 414;
        fontptr->letter[69].width =  12;
        fontptr->letter[70].offset = 426;
        fontptr->letter[70].width =  11;
        fontptr->letter[71].offset = 437;
        fontptr->letter[71].width =  15;
        fontptr->letter[72].offset = 452;
        fontptr->letter[72].width =  14;
        fontptr->letter[73].offset = 466;
        fontptr->letter[73].width =  5;
        fontptr->letter[74].offset = 471;
        fontptr->letter[74].width =  12;
        fontptr->letter[75].offset = 483;
        fontptr->letter[75].width =  15;
        fontptr->letter[76].offset = 498;
        fontptr->letter[76].width =  12;
        fontptr->letter[77].offset = 510;
        fontptr->letter[77].width =  19;
        fontptr->letter[78].offset = 529;
        fontptr->letter[78].width =  16;
        fontptr->letter[79].offset = 545;
        fontptr->letter[79].width =  15;
        fontptr->letter[80].offset = 560;
        fontptr->letter[80].width =  13;
        fontptr->letter[81].offset = 573;
        fontptr->letter[81].width =  16;
        fontptr->letter[82].offset = 589;
        fontptr->letter[82].width =  14;
        fontptr->letter[83].offset = 603;
        fontptr->letter[83].width =  14;
        fontptr->letter[84].offset = 617;
        fontptr->letter[84].width =  14;
        fontptr->letter[85].offset = 631;
        fontptr->letter[85].width =  14;
        fontptr->letter[86].offset = 645;
        fontptr->letter[86].width =  15;
        fontptr->letter[87].offset = 660;
        fontptr->letter[87].width =  24;
        fontptr->letter[88].offset = 684;
        fontptr->letter[88].width =  16;
        fontptr->letter[89].offset = 700;
        fontptr->letter[89].width =  15;
        fontptr->letter[90].offset = 715;
        fontptr->letter[90].width =  14;





// SMALLCAPS TEXT ...

// Allocate memory and load in data ...

		fontptr = &myfonttype[SMALLCAPS]; 
		
//		fontptr->picptr = unpackpcx( "fonts\\smallcaps.pcx" );
		fontptr->surfaceno = DDS_SMALLCAPSFONT;

		fontptr->height = 10;
		fontptr->width = 400;


        fontptr->letter[0].offset = 0;
        fontptr->letter[0].width = 4;

        fontptr->letter[1].offset = 249;
        fontptr->letter[1].width =  5;
        fontptr->letter[2].offset = 254;
        fontptr->letter[2].width =  5;
        fontptr->letter[3].offset = 259;
        fontptr->letter[3].width =  7;
        fontptr->letter[4].offset = 266;
        fontptr->letter[4].width =  7;
        fontptr->letter[5].offset = 273;
        fontptr->letter[5].width =  10;
        fontptr->letter[6].offset = 283;
        fontptr->letter[6].width =  7;
        fontptr->letter[7].offset = 290;
        fontptr->letter[7].width =  3;
        fontptr->letter[8].offset = 293;
        fontptr->letter[8].width =  5;
        fontptr->letter[9].offset = 298;
        fontptr->letter[9].width =  5;
        fontptr->letter[10].offset = 303;
        fontptr->letter[10].width =  7;
        fontptr->letter[11].offset = 310;
        fontptr->letter[11].width =  5;
        fontptr->letter[12].offset = 315;
        fontptr->letter[12].width =  4;
        fontptr->letter[13].offset = 319;
        fontptr->letter[13].width =  5;
        fontptr->letter[14].offset = 324;
        fontptr->letter[14].width =  4;
        fontptr->letter[15].offset = 328;
        fontptr->letter[15].width =  5;

        fontptr->letter[16].offset = 182;
        fontptr->letter[16].width =  7;
        fontptr->letter[17].offset = 189;
        fontptr->letter[17].width =  5;
        fontptr->letter[18].offset = 194;
        fontptr->letter[18].width =  7;
        fontptr->letter[19].offset = 201;
        fontptr->letter[19].width =  7;
        fontptr->letter[20].offset = 208;
        fontptr->letter[20].width =  6;
        fontptr->letter[21].offset = 214;
        fontptr->letter[21].width =  7;
        fontptr->letter[22].offset = 221;
        fontptr->letter[22].width =  7;
        fontptr->letter[23].offset = 228;
        fontptr->letter[23].width =  7;
        fontptr->letter[24].offset = 235;
        fontptr->letter[24].width =  7;
        fontptr->letter[25].offset = 242;
        fontptr->letter[25].width =  7;

        fontptr->letter[26].offset = 333;
        fontptr->letter[26].width =  3;
        fontptr->letter[27].offset = 336;
        fontptr->letter[27].width =  4;
        fontptr->letter[28].offset = 340;
        fontptr->letter[28].width =  5;
        fontptr->letter[29].offset = 345;
        fontptr->letter[29].width =  5;
        fontptr->letter[30].offset = 350;
        fontptr->letter[30].width =  5;
        fontptr->letter[31].offset = 355;
        fontptr->letter[31].width =  7;
        fontptr->letter[32].offset = 362;
        fontptr->letter[32].width =  8;

        fontptr->letter[33].offset = 0;
        fontptr->letter[33].width =  7;
        fontptr->letter[34].offset = 7;
        fontptr->letter[34].width =  7;
        fontptr->letter[35].offset = 14;
        fontptr->letter[35].width =  7;
        fontptr->letter[36].offset = 21;
        fontptr->letter[36].width =  7;
        fontptr->letter[37].offset = 28;
        fontptr->letter[37].width =  6;
        fontptr->letter[38].offset = 34;
        fontptr->letter[38].width =  6;
        fontptr->letter[39].offset = 40;
        fontptr->letter[39].width =  7;
        fontptr->letter[40].offset = 47;
        fontptr->letter[40].width =  7;
        fontptr->letter[41].offset = 54;
        fontptr->letter[41].width =  5;
        fontptr->letter[42].offset = 59;
        fontptr->letter[42].width =  7;
        fontptr->letter[43].offset = 66;
        fontptr->letter[43].width =  7;
        fontptr->letter[44].offset = 73;
        fontptr->letter[44].width =  6;
        fontptr->letter[45].offset = 79;
        fontptr->letter[45].width =  9;
        fontptr->letter[46].offset = 88;
        fontptr->letter[46].width =  8;
        fontptr->letter[47].offset = 96;
        fontptr->letter[47].width =  7;
        fontptr->letter[48].offset = 103;
        fontptr->letter[48].width =  7;
        fontptr->letter[49].offset = 110;
        fontptr->letter[49].width =  7;
        fontptr->letter[50].offset = 117;
        fontptr->letter[50].width =  7;
        fontptr->letter[51].offset = 124;
        fontptr->letter[51].width =  7;
        fontptr->letter[52].offset = 131;
        fontptr->letter[52].width =  7;
        fontptr->letter[53].offset = 138;
        fontptr->letter[53].width =  7;
        fontptr->letter[54].offset = 145;
        fontptr->letter[54].width =  7;
        fontptr->letter[55].offset = 152;
        fontptr->letter[55].width =  9;
        fontptr->letter[56].offset = 161;
        fontptr->letter[56].width =  7;
        fontptr->letter[57].offset = 168;
        fontptr->letter[57].width =  7;
        fontptr->letter[58].offset = 175;
        fontptr->letter[58].width =  7;

        fontptr->letter[59].offset = 370;
        fontptr->letter[59].width =  4; // SPECIAL
        fontptr->letter[60].offset = 374;
        fontptr->letter[60].width =  5;
        fontptr->letter[61].offset = 379;
        fontptr->letter[61].width =  4; // SPECIAL
        fontptr->letter[62].offset = 383;
        fontptr->letter[62].width =  7;
        fontptr->letter[63].offset = 390;
        fontptr->letter[63].width =  5;
        fontptr->letter[64].offset = 395;
        fontptr->letter[64].width =  4;


        fontptr->letter[65].offset = 0;
        fontptr->letter[65].width =  7;
        fontptr->letter[66].offset = 7;
        fontptr->letter[66].width =  7;
        fontptr->letter[67].offset = 14;
        fontptr->letter[67].width =  7;
        fontptr->letter[68].offset = 21;
        fontptr->letter[68].width =  7;
        fontptr->letter[69].offset = 28;
        fontptr->letter[69].width =  6;
        fontptr->letter[70].offset = 34;
        fontptr->letter[70].width =  6;
        fontptr->letter[71].offset = 40;
        fontptr->letter[71].width =  7;
        fontptr->letter[72].offset = 47;
        fontptr->letter[72].width =  7;
        fontptr->letter[73].offset = 54;
        fontptr->letter[73].width =  5;
        fontptr->letter[74].offset = 59;
        fontptr->letter[74].width =  7;
        fontptr->letter[75].offset = 66;
        fontptr->letter[75].width =  7;
        fontptr->letter[76].offset = 73;
        fontptr->letter[76].width =  6;
        fontptr->letter[77].offset = 79;
        fontptr->letter[77].width =  9;
        fontptr->letter[78].offset = 88;
        fontptr->letter[78].width =  8;
        fontptr->letter[79].offset = 96;
        fontptr->letter[79].width =  7;
        fontptr->letter[80].offset = 103;
        fontptr->letter[80].width =  7;
        fontptr->letter[81].offset = 110;
        fontptr->letter[81].width =  7;
        fontptr->letter[82].offset = 117;
        fontptr->letter[82].width =  7;
        fontptr->letter[83].offset = 124;
        fontptr->letter[83].width =  7;
        fontptr->letter[84].offset = 131;
        fontptr->letter[84].width =  7;
        fontptr->letter[85].offset = 138;
        fontptr->letter[85].width =  7;
        fontptr->letter[86].offset = 145;
        fontptr->letter[86].width =  7;
        fontptr->letter[87].offset = 152;
        fontptr->letter[87].width =  9;
        fontptr->letter[88].offset = 161;
        fontptr->letter[88].width =  7;
        fontptr->letter[89].offset = 168;
        fontptr->letter[89].width =  7;
        fontptr->letter[90].offset = 175;
        fontptr->letter[90].width =  7;







} // end inittext


