/*
define.h
constants and objects for Matchblox

Copyright (C) 1995-2005  Everett Bell

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __MYDEFINE
#define __MYDEFINE

#include <windows.h>

#include "resource.h"



#define DEBUG
#define EDITORS


// Game Types ...
#define BUMPIT		0
#define BUMPIT3		1
#define BUMPIT4		2
#define PUZZLE		3
#define PUSHIT		4
#define USERPLAY	5
#define USEREDIT	6



#define NUMSURFACES		17
#define NUMOFFONTS		3


#define CHECKWIN		1
#define CHECKLOSE		2

// Language offsets ...
#define ENGLISH			0
#define GERMAN			100
#define SPANISH			200
#define FRENCH			300



// Text colors ...
#define TEXTRED			0
#define TEXTORANGE		1
#define TEXTYELLOW		2
#define TEXTGREEN		3
#define TEXTCYAN		4
#define TEXTBLUE		5
#define TEXTPURPLE		6
#define TEXTWHITE		7


// Font types ...

#define EURO			0
#define EURODARK		1
#define SMALLCAPS		2


// Font Surfaces ...
#define DDS_BLOCKS			1
#define DDS_EUROFONT		2
#define DDS_EUROFONTDARK	3
#define DDS_SMALLCAPSFONT	4

#define DDS_COPYRIGHT		5
#define DDS_AD				6
#define DDS_NEWGAME			7
#define DDS_PLAYBG2			8
#define DDS_PLAYBG3			9
#define DDS_PLAYBG4			10
#define DDS_PUZZLES			11
#define DDS_NSPGAMES		12

#define DDS_MATCH2			13
#define DDS_MATCH3			14
#define DDS_MATCH4			15

#define DDS_SLIDERS			16



#define byte	unsigned char
#define word	unsigned short int
#define dword	unsigned int


#define SCREENWIDTH		800
#define SCREENHEIGHT	600
#define COLORDEPTH		16
#define VSCREENWIDTH	800
#define VSCREENHEIGHT	600
#define VSOFFSET		0

#define VSLEFT		0
#define VSTOP		0
#define VSRIGHT		800
#define VSBOTTOM	600



// Game Screens ...
#define GS_MAINMENU			1
#define GS_OKBOX			2
#define GS_CONFIRM			3
#define GS_SELECTLIST		4

#define GS_PLAY				5
#define GS_ARRANGEPUZZLES	7



#define WINGAME			4

// Exit Errors ...

#define OK              0
#define OUTOFMEM        1
#define NOVIDEO         2
#define NOFILE          3
#define NOSOUND         4
#define NOMOUSE         5
#define MISC            6



// Character moods ...
#define NORMAL			0
#define HAPPY			1


// Cursor Values ...

#define CURSORANIMSPEED	4

// Cursor modes ...

#define CURSOROFF       0
#define MENUCURSOR		1


#define SPRITEONCE	0	// Used in showspriteonce()

// #define MENUBORDER  56

// Button function macros ...

#define BUTTONRETURN          0
#define NOTHING               1
#define SLIDER                2
#define CHECKBOX              3
#define RADIO1                4
#define RADIO2                5
#define RADIO3                6
#define RADIO4                7
#define RADIO5                8
#define RADIO6                9
#define RADIO7                10
#define RADIO8                11

// Offset into button BMP of depressed version
// (should change for different height buttons) ...
#define BUTTONOFF       512L*24L


// 16bit stuff ...
// Pixel formats ...
#define PF_555		1
#define PF_565		2
#define PF_OTHER	3






// Function MACROS ...

/*
* For 16 bit only!
*

// Sets colors ... IN 555 format!
#define RGB16(r,g,b) (word)((r<<10)|(g<<5)|(b))
#define GETR16(color) (((word)(color & 0x7C00))>>10)
#define GETG16(color) (((word)(color & 0x03E0))>>5)
#define GETB16(color) (word)(color & 0x001F)

#define SETTEXTCOLOR(r,g,b) { { textr = r; if( pixelformat==PF_555 ) textg = g; else { textg = g<<1; }; textb = b; } }

#define SETSELECTCOLOR(r,g,b) { selectcolorr = r; selectcolorg = g; selectcolorb = b; }
#define SETSELECTCOLORBASE(r,g,b) { selectcolorbaser = r; selectcolorbaseg = g; selectcolorbaseb = b; }

*/

#define times768(x) (((x)<<9)+((x)<<8))
#define times640(x) (((x)<<9)+((x)<<7))


// Locks Virtual Screen ...
#define LOCKVS if( !indrawscreen ) lockvs();
#define UNLOCKVS if( !indrawscreen ) unlockvs();



// Debugging ...
#define TIMESTAMP			{ sprintf( debugstr2, "%.8d - GS: %.2d,  ", tickcount, gamescreen ); WriteFile( debugfh, debugstr2, strlen(debugstr2), &numofbytesread, NULL ); }
#define WRITETODEBUGFILE	{ TIMESTAMP; WriteFile( debugfh, debugstr, strlen(debugstr), &numofbytesread, NULL ); }



// Sounds ...
/*
#define CLICK				if( soundon ) lpdsbuffer[SND_CLICK]->Play( NULL, NULL, NULL )
#define CLICK1				if( soundon ) lpdsbuffer[SND_CLICK1]->Play( NULL, NULL, NULL )
#define CLICK2				if( soundon ) lpdsbuffer[SND_CLICK2]->Play( NULL, NULL, NULL )
#define CLICK3				if( soundon ) lpdsbuffer[SND_CLICK3]->Play( NULL, NULL, NULL )
#define CLICK4				if( soundon ) lpdsbuffer[SND_CLICK4]->Play( NULL, NULL, NULL )
#define PLAYWINGAME			if( soundon ) lpdsbuffer[SND_WINGAME]->Play( NULL, NULL, NULL )
#define PLAYMATCH(x)		if( soundon ) lpdsbuffer[SND_2-1+x]->Play( NULL, NULL, NULL )
#define PLAYBUTTONOVER		if( soundon ) lpdsbuffer[SND_10]->Play( NULL, NULL, NULL )

*/

#define CLICK				PlaySound( "SND_CLICK", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define CLICK1				PlaySound( "SND_CLICK1", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define CLICK2				PlaySound( "SND_CLICK2", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define CLICK3				PlaySound( "SND_CLICK3", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define CLICK4				PlaySound( "SND_CLICK4", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define PLAYWINGAME			PlaySound( "SND_WINGAME", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define PLAYLOSEGAME		PlaySound( "SND_LOSEGAME", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );
#define PLAYMATCH(x)		{ sprintf( tempstr, "SND_%d", x+1 ); PlaySound( tempstr, GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC ); }
#define PLAYBUTTONOVER		PlaySound( "SND_10", GetModuleHandle(NULL), SND_RESOURCE|SND_ASYNC );




// Structures ...

typedef struct pcxheader_typ {
	char manufacturer;
    char version;
    char encoding;
    char bitsperpixel;
    short int x, y;                       // Upper left corner
    short int width, height;              // Size of image
    short int horzres;
    short int vertres;
    char egapalette[48];
    char reserved;
    char numcolorplanes;
    short int bytesperline;
    short int palettetype;
    char padding[58];

} PCXHEADER;


typedef struct sprite_typ
{
	word *picptr;
    word *bgptr;
    short int height, width;
    short int x, y;
    byte visible;

    byte beingused;

	byte blendamt;

} SPRITE;

typedef struct animsprite_typ
{
	word *picptr[32];
    word *bgptr;
    short int height, width;
    short int x, y;
    byte visible;
    byte animating;
    byte frame;

    byte animspeed;
    unsigned int clockmax;

    byte numframes;

    byte beingused;

	byte blendamt;

} ANIMSPRITE;

typedef struct letter_typ
{
	short int offset;
	byte width;
} LETTER;

typedef struct myfont_typ
{
	LETTER letter[96];
	byte *picptr;
	byte height;
	int width;
	int surfaceno;
} MYFONT;

typedef struct button_typ
{
    int x1, y1;
    int width;
    int height;
    char pushed;
    char name[32];
    word color;
    char function;          // Macro for what button does when clicked
    int variable;           // Variable to be applied with function (i.e. add)
    int value;
    char *memptr;           // Optional pointer to function call
    char glowing;           // Flag for when mouse is over button
    int offset;
    byte page;

} BUTTON;




typedef struct selectoption_typ
{
	char text[64];
	byte disabled;
} SELECTOPTION;



typedef struct floatingnumber_typ
{
	int amount;	// Value for score
	byte color; // Which block color score came from (10 possible)

	byte active; // Whether active or not

	int x;	// Current screen location
	int y;

	int pathx[8];
	int pathy[8];

} FLOATINGNUMBER;	// For special FX when player scores


struct CHECKNODE
{
	int x;	// Board location
	int y;
	byte color;	// Block color (1-10)
	int childno[4];
	int parentno;
	byte numofchildren;

};	// Used in checkboard() to look for matches



// End one time only definitions ...
#endif




// Function Declarations



// DirectSound ...

#define NUMEVENTS	8
#define NUMSOUNDS	16

#define SND_1		1
#define SND_2		2
#define SND_3		3
#define SND_4		4
#define SND_5		5
#define SND_6		6
#define SND_7		7
#define SND_8		8
#define SND_9		9
#define SND_10		10
#define SND_11		11
#define SND_12		12
#define SND_13		13
#define SND_14		14
#define SND_15		15

#define SND_CLICK		1

#define SND_WINGAME		11
#define SND_CLICK1		12
#define SND_CLICK2		13
#define SND_CLICK3		14
#define SND_CLICK4		15



BOOL LoadSound( dword soundno, char *lpszfilename );

BOOL InitDSound(HWND hwnd, GUID *pguid);
BOOL SetupStreamBuffer(LPSTR lpzFileName);
BOOL StreamToBuffer(DWORD dwPos);
void DSExit(void);

/*
// DirectMusic
IDirectMusicPerformance* CreatePerformance(void);
IDirectMusicLoader* CreateLoader(void);
IDirectMusicSegment* LoadSegment(IDirectMusicLoader* pLoader, char *filename);
*/











// Dialog Box functions ...
BOOL CALLBACK hiscoredialogfunc( HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam );
BOOL CALLBACK showpassworddialogfunc( HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam );
BOOL CALLBACK enterpassworddialogfunc( HWND hdwnd, UINT message, WPARAM wParam, LPARAM lParam );


int selectlist( void );
void addtoselectlist( char *text, byte disabled );

void waitforinput( void );
void systemmessage( char *textptr );


BOOL lockvs( void );
BOOL unlockvs( void );


BOOL lockbackbuffer( void );
BOOL unlockbackbuffer( void );


// Message Handlers for various gamescreens ...
int mainmenumessagehandler( UINT message, WPARAM wParam, LPARAM lParam );
int okboxmessagehandler( UINT message, WPARAM wParam, LPARAM lParam );
int confirmmessagehandler( UINT message, WPARAM wParam, LPARAM lParam );

int waitmessagehandler( UINT message, WPARAM wParam, LPARAM lParam );
int selectlistmessagehandler( UINT message, WPARAM wParam, LPARAM lParam );

int playmessagehandler( UINT message, WPARAM wParam, LPARAM lParam );
int arrangepuzzlesmessagehandler( UINT message, WPARAM wParam, LPARAM lParam );


BOOL refreshscreen( void );

byte pushgamescreen( byte gs );
byte popgamescreen( void );


long FAR PASCAL WindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
long FAR PASCAL AdWindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );


void showpixelformat( void );

void finiObjects( void );


word *loadandunpackbmp( char *tempfilename );


void initspritefrombmp( byte spriteno, char *filename );

// IDirectDrawSurface4 *DDLoadBitmapSYS(IDirectDraw4 *pdd, LPCSTR szBitmap, int dx, int dy);

void ShowDDError(char *where, HRESULT hErr);

HRESULT restoreAll( void );

void clearscreen( void );
void clearvscreen( void );
void blankscreen( void );



void CALLBACK mytimer(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2);

inline void fastblt( void );



/////////////////////////
// OLD PROTOTYPES !!!! //
/////////////////////////

byte confirm( char *text );
void okbox( char *textptr );


int loadfile( char *path, char *par );

void mousemessage( char *textptr );


void drawwindow( int x, int y, int width, int height );

void wait( void );
void waitfor( int numticks );

void screenshot( void );


// Sprite functions ...

void initsprite( byte spriteno, short int width, short int height );
void deletesprite( byte spriteno );
void showsprite( byte spriteno );
void showspriteonce( char *tempfilename, int x, int y );
void hidesprite( byte spriteno );
void animatesprites( byte numsprites );
void initanimsprite( byte spriteno, short int width, short int height, byte numframes,
                      byte speed, byte animating );
void deleteanimsprite( byte spriteno );
void showanimsprite( byte spriteno );
void hideanimsprite( byte spriteno );

void scalesprite( byte srcsprite, byte destsprite, int xsize, int ysize );



void cleanup( char errortype, char *text );


void initscreen( void );
void initgraphics( void );
void drawscreen( void );

void displayscreen( void );


void showcursor( void );
byte *unpackpcx( char *filename );
void showbitmap( char *tempfilename, int x, int y );


void setpalette(char *Palette);
void setpalfrompcx( char *filename );


void inittext( void );
int getlength( char *text );
void showchar( char c, int x, int y, char recessed );
void showtext( char *text, int x, int y );
void centertext( char *text, int y, int min, int max );
void showtextjr( char *text, int x, int y );

void showtextfile( char *name );

void mouse_setpos( int x, int y );
void updatemouse( void );
void showcursor( void );
void hidecursor( void );
void setcursormode( byte mode );
void animatecursor( void );   // Set up as an interrupt if not too slow

/*
void initgraphics( void );
void drawbutton( char buttonno );
byte popupbox( void );
*/



void drawbutton( char buttonno );


void playgame( void );
void drawboard( byte outline );

int checkboard( void );


void drawbg( void );
void drawscore( void );
void drawbuttons( void );
void drawsliderbuttons( void );

int getdistance( int x1, int y1, int x2, int y2 );

void initdistance( void );


int getchildren( int nodeno );


int checkspace( int x, int y );
void removeblock( int nodeno );


void showhiscores( void );
void showgametypes( void );
void showquitbutton( void );


void intro( void );

void CALLBACK mytimer( UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2 );


int makenewtile( void );
int pushtiles( void );


void drawreservoir( void );

int loadpuzzles( char *filename );
int savepuzzles( char *filename );


void placeblock( byte blocknumber );


void drawallpuzzles( byte outline );
int arrangepuzzles( void );

void drawbuttonsonbg( void );

void dopuzzlehiscore( void );
void showpassword( void );
void enterpassword( void );



// Cydoor ...
int openadwindow( void );
int closeadwindow( void );



void checkcursor( void );
void checkmenucursor( void );

BOOL nomorematches( void );


int saveuserboards( void );
int loaduserboards( void );
